var LibReg = function()
{
	var libreg = {};
	var debug = 0;
	
	var libCommentsIsValid = false;
	
	// Page initialization
	libreg.initPage = function()
	{
		// Step 1
		Events.addEventHandler('bioplate', 'change', libreg.bioPlateOnChange);
		Events.addEventHandler('outcomeSuccess', 'click', libreg.outcomeOnChange);
		Events.addEventHandler('outcomeFailed', 'click', libreg.outcomeOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', libreg.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', libreg.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', libreg.validateStep2);
		Events.addEventHandler('dilutionDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('neoPrepDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('libVolume', 'change', Wizard.validateNumber);
		Events.addEventHandler('libVolume', 'keypress', Events.numberOnly);
		Events.addEventHandler('libMolarity', 'change', Wizard.validateNumber);
		Events.addEventHandler('libMolarity', 'keypress', Events.numberOnly);
		Events.addEventHandler('libSize', 'change', Wizard.validateNumber);
		Events.addEventHandler('libSize', 'keypress', Events.numberOnly);
		Events.addEventHandler('libComments', 'blur', libreg.libCommentsOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', libreg.submit);
		
		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLibraryPlatesForLibPrep';
		url += '&plateType=NEOPREP';
		Wizard.showLoadingAnimation('Loading NeoPrep plates...');
		Wizard.asyncJsonRequest(url, libreg.initializeStep1);
	}


	libreg.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var bioplates = response.bioplates;

		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				var name = bioplate.name;
				if (bioplate.AutoProcessing == 'PreNormalizeRNA')
				{
					name += ' (pre-normalized)';
				}
				else if (bioplate.DilutionDate)
				{
					name += ' (diluted ' + Reggie.reformatDate(bioplate.DilutionDate) + ')';
				}
				var option = new Option(name, bioplate.id);
				option.bioplate = bioplate;
				plates.options[plates.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No NeoPrep bioplates available for processing.');
			return;
		}

		libreg.bioPlateOnChange();
		libreg.outcomeOnChange();
		frm.bioplate.focus();

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	libreg.bioPlateOnChange = function()
	{
		var frm = document.forms['reggie'];
		var selectedPlate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;
		Wizard.setInputStatus('bioplate', 'valid');
	}

	libreg.outcomeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var failed = Doc.element('outcomeFailed').checked
	
		if (failed)
		{
			Doc.addClass(frm.libComments, 'required');
			Doc.hide('filesSection');
		}
		else
		{
			Doc.removeClass(frm.libComments, 'required');
			Doc.show('filesSection');
		}
	}

	
	libreg.validateStep1 = function(event)
	{}
	
	libreg.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];	

		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLibraryInfoForPlate&bioplate='+frm.bioplate.value;
		url += '&loadDilutionInfo=1';
		Wizard.showLoadingAnimation('Loading NeoPrep plate information...');
		Wizard.asyncJsonRequest(url, libreg.libraryInfoLoaded);
	}
	
	libreg.libraryInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];	
		var failed = Doc.element('outcomeFailed').checked;

		if (failed)
		{
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of failure.');
			libreg.libCommentsOnChange();
			Doc.hide('lib-section');
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		var plate = response.plate;
		var libs = response.libraries;
		
		// Pre-process the return mRNA items
		var allPreNormalized = true;
		for (var i = 0; i < libs.length; i++)
		{
			allPreNormalized &= libs[i].rna.preNormalized;
		}
		
		if (plate.DilutionDate)
		{
			frm.dilutionDate.value = plate.DilutionDate;
			frm.dilutionOperator.value = Strings.encodeTags(plate.DilutionOperator);
		}
		if (allPreNormalized)
		{
			frm.dilutionDate.disabled = true;
			frm.dilutionOperator.disabled = true;
			frm.neoPrepDate.focus();
			Wizard.setInputStatus('dilutionDate', '', 'PreNormalized');
		}
		else
		{
			frm.dilutionDate.focus();
		}

		// Load Library protocols
		Reggie.loadProtocols('LIBRARY_PROTOCOL', 'libProtocol');
		if (!failed)
		{
			Events.sendChangeEvent('libVolume');
			Events.sendChangeEvent('libMolarity');
			Events.sendChangeEvent('libSize');
		}
	}
	
	libreg.libVolumeOnChange = function(event)
	{
		var frm = document.forms['reggie'];

		var target = event.currentTarget;
		libVolumeIsValid = false;
		Wizard.setInputStatus(target.id);
		
		var defaultValue = Data.int(target, 'default-value');
		var value = target.value;
		if (!defaultValue) Data.set(target, 'default-value', value); // Initalize the default value

		if (value == '')
		{
			Wizard.setInputStatus(target.id, 'invalid', 'Missing');
			return;
		}
		
		if (!Numbers.isInteger(value))
		{
			Wizard.setInputStatus(target.id, 'invalid', 'Not a number');
			return;
		}
		
		libVolumeIsValid = true;
		if (parseInt(value) != defaultValue)
		{
			Wizard.setInputStatus(target.id, 'warning', 'Default: ' + defaultValue);
		}
		else
		{
			Wizard.setInputStatus(target.id, 'valid');
		}

	}
	
	libreg.libCommentsOnChange = function()
	{
		var failed = Doc.element('outcomeFailed').checked;
		if (!failed) return;

		var frm = document.forms['reggie'];
		libCommentsIsValid = false;
		Wizard.setInputStatus('libComments');
		
		var comments = frm.libComments.value;
		if (comments == '')
		{
			Wizard.setInputStatus('libComments', 'invalid', 'Missing');
			return;
		}

		Wizard.setInputStatus('libComments', 'valid');
		libCommentsIsValid = true;
	}

	libreg.validateStep2 = function(event)
	{
		var valid = true;
		
		valid &= Wizard.isValid('dilutionDate');
		valid &= Wizard.isValid('neoPrepDate');
		
		var failed = Doc.element('outcomeFailed').checked;
		if (failed)
		{
			valid &= libCommentsIsValid;
		}
		else
		{
			valid &= Wizard.isValid('libVolume');
			valid &= Wizard.isValid('libMolarity');
			valid &= Wizard.isValid('libSize');
		}
		
		if (!valid) event.preventDefault();
	}

	
	libreg.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		submitInfo.bioplate = parseInt(frm.bioplate.value, 10);

		var failed = Doc.element('outcomeFailed').checked;
		submitInfo.failed = failed;
		
		if (!frm.dilutionDate.disabled)
		{
			submitInfo.dilutionDate = frm.dilutionDate.value;
			submitInfo.dilutionOperator = frm.dilutionOperator.value;
		}
		submitInfo.neoPrepDate = frm.neoPrepDate.value;
		submitInfo.neoPrepOperator = frm.neoPrepOperator.value;
		
		if (!failed)
		{
			submitInfo.libVolume = parseFloat(frm.libVolume.value, 10);
			submitInfo.libMolarity = parseFloat(frm.libMolarity.value, 10);
			submitInfo.libSize = parseFloat(frm.libSize.value, 10);
		}
		
		submitInfo.libProtocol = parseInt(frm.libProtocol.value, 10);
		submitInfo.libComments = frm.libComments.value;
		
		var	url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterNeoPrepLibraries';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, libreg.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	libreg.submissionResults = function(response)
	{
		var frm = document.forms['reggie'];
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	
	return libreg;
}();

Doc.onLoad(LibReg.initPage);

